clc; clear;

% Load the DJIA data implied vols
IV = [...
    19.62	19.47	20.19	21.15;
    19.10	19.05	19.80	20.82;
    18.60	18.61	19.43	20.57;
    18.10	18.12	19.07	20.21;
    17.61	17.74	18.71	20.00;
    17.18	17.43	18.42	19.74;
    16.71	17.06	18.13	19.50;
    16.44	16.71	17.83	19.27;
    16.45	16.41	17.60	18.99;
    16.61	16.25	17.43	18.84;
    17.01	16.02	17.26	18.62;
    17.55	16.10	17.16	18.46;
    17.96	16.57	17.24	18.42]./100;

% Strikes and maturities
K = (124:136);
T = [37 72 135 226]./365;
[NK NT] = size(IV);
N = NK*NT;

% Arrange IV into a vector
z = IV(:);

% Arrange strikes into a vector
Temp = meshgrid(K);
x = Temp(1:NT,:);
x = x(:);

% Arrange maturities into a vector
y = repmat(T,1,NK)';

% Robust estimate of the sample standard deviation
sigmaX = median(abs(x-median(x)))/0.6745;
sigmaY = median(abs(y-median(y)))/0.6745;
sigmaZ = median(abs(z-median(z)))/0.6745;

% 0ptimal bandwidth from Bowman and Azzalini (1997) p.31
hx = sigmaX*(1/N)^(1/6);
hy = sigmaY*(1/N)^(1/6);
hz = sigmaZ*(1/N)^(1/6);
hK = sqrt(hx*hz);
hT = sqrt(hy*hz);

% Vector of N-W implied vols
for k=1:N
    vec(k) = NadarayaWatson(x(k),y(k),x,y,z,hK,hT);
end

% Convert to surface
IVi = zeros(NK,NT);
IVi(:) = vec';

% Surface of the result
subplot(1,2,1)
surf(T,K,IV)
subplot(1,2,2)
surf(T,K,IVi)

